import click
import os
import json
import sqlite_utils
import requests
import dateutil.parser


@click.group()
@click.version_option()
def cli():
    "Save data from Pinboard to a SQLite database"


@cli.command()
@click.option(
    "-a",
    "--auth",
    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False),
    default="auth.json",
    help="Path to save token to",
    show_default=True,
)
def auth(auth):
    "Save authentication credentials to a JSON file"
    click.echo("Find your API token here: https://pinboard.in/settings/password")
    click.echo(
        "Paste the whole thing including your username "
        "(e.g. yourname:xxxyyyzzz) below."
    )
    click.echo()
    pinboard_token = click.prompt("API Token")
    auth_data = json.load(open(auth)) if os.path.exists(auth) else {}
    auth_data["pinboard_token"] = pinboard_token
    json.dump(auth_data, open(auth, "w"))


@cli.command()
@click.argument(
    "database",
    required=True,
    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False),
)
@click.option(
    "-a",
    "--auth",
    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False, exists=True),
    default="auth.json",
    help="Path to read auth token from",
    show_default=True,
)
@click.option(
    "--since",
    is_flag=True,
    default=False,
    help="Pull new posts since last saved post in DB",
)
@click.option("--since-date", metavar="DATE", help="Pull new posts since DATE")
def posts(database, auth, since, since_date):
    if since and since_date:
        raise click.UsageError("use either --since or --since-date, not both")

    token = json.load(open(auth))["pinboard_token"]
    params = {"format": "json", "auth_token": token}

    db = sqlite_utils.Database(database)

    if since and db["posts"].exists:
        since_date = db.conn.execute("SELECT max(time) FROM posts;").fetchone()[0]
    if since_date:
        params["fromdt"] = (
            dateutil.parser.parse(since_date)
            .replace(microsecond=0, tzinfo=None)
            .isoformat()
            + "Z"
        )

    posts = requests.get(f"https://api.pinboard.in/v1/posts/all", params=params).json()
    _save_posts(db, posts)


def _save_posts(db, posts):
    # Convert/coerce some fields
    for post in posts:
        post["shared"] = post["shared"] == "yes"
        post["toread"] = post["toread"] == "yes"
        post["time"] = dateutil.parser.parse(post["time"])
        post["tags"] = json.dumps(post["tags"].split())

    db["posts"].upsert_all(
        posts,
        pk="hash",
        column_order=[
            "hash",
            "href",
            "description",
            "extended",
            "meta",
            "time",
            "shared",
            "toread",
            "tags",
        ],
    )


# TODO: notes?



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pinboard_to_sqlite']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.0,<8.0',
 'python-dateutil>=2.8.1,<3.0.0',
 'requests>=2.22.0,<3.0.0',
 'sqlite-utils>=1.12.1,<2.0.0']

entry_points = \
{'console_scripts': ['pinboard-to-sqlite = pinboard_to_sqlite.cli:cli']}

setup_kwargs = {
    'name': 'pinboard-to-sqlite',
    'version': '1.2.0',
    'description': 'Save data from Pinboard to a SQLite database',
    'long_description': "# Save data from Pinboard to a SQLite database.\n\nInspired by (and using libraries from) [Simon Willison's Dogsheep\nproject](https://github.com/dogsheep). You're probably going to want to run\n[Datasette](https://github.com/simonw/datasette) on the resulting db.\n\n## How to install\n\n```\n$ pip install pinboard-to-sqlite\n```\n\n## Authentication\n\nRun:\n\n```\n$ pinboard-to-sqlite auth\n```\n\nThis will direct you to https://pinboard.in/settings/password to find your API\ntoken, which you'll then paste into the terminal. This'll get saved in an\n`auth.json` file, which subsequent commands will pick up.\n\nTo save to a different file, see the `-a` / `--auth` flag.\n\n## Fetching posts\n\nRun:\n\n```\n$ pinboard-to-sqlite posts pinboard.db\n```\n\nWhere `pinboard.db` is the name of the database you'd like to save posts to.\nNote that the API this uses has a rate limit of once per minute, so don't run\nthis command more than once per minute (I don't know why you would). This\ndoesn't seem to be enforced fairly loosely, but be careful anyway.\n",
    'author': 'Jacob Kaplan-Moss',
    'author_email': 'jacob@jacobian.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jacobian/pinboard-to-sqlite/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
